/*
m_loop_play - Play/loop a sample

DESCRIPTION
Play/loop a sample with pitch control, number of repeats (or infinite looping),
and a specific portion of the sample can be selected.

SYNTAX
aout m_loop_play Sname, kpitch [, imode] [, irepeats] [, ibegin[ [, iend]

INITIALIZATION
imode - Loop mode:
	0: forward
	1: backward
	2: forward -> backward
irepeats - Number of repeats, 0 means infinte looping, otherwise repeat till
	the number of repititions is reached or the end of the note, whichever
	happens first.
ibegin - Where to begin playing the sample in seconds (default=0)
iend - Where to stop playing the sample (default=-1, i.e. the end)

PERFORMANCE
aout - Audio output
kpitch - Pitch factor, 1=normal speed, <1 slower, >1 faster

CREDITS
Author: Jeanette C.
*/

opcode m_loop_play, a, Skoooj
	Sname, kPitch, iMode, iRepeat, iBegin, iEnd xin

	setksmps 1 ; because of the syncphasor

	; Set up the basic channel name
	SchannelName strcat "m_loop.", Sname

	; Initialise the audio output
	aOut init 0

	kCounter init 0 ; loop counter
	aSyncIn init 0  ; for the sync phasor
	aSyncOut init 0

	; Increase the counter
	if k(aSyncOut) == 1 then
		kCounter += 1
	endif

	if chnget(strcat(SchannelName, ".Valid")) != 1 then
		prints "\nWARNING m_loop_play %s: the loop isn't valid. Was it deleted?", Sname
	else
		; Get the table number and table information
		ifn chnget strcat(SchannelName, ".Table")
		iLength = ftlen(ifn)

		; Calculate the start and end in samples
		if (iBegin * sr) >= iLength then
			prints "\nWARNING m_loop_play %s: loop start too late, setting to 0", Sname
			iBeginReal = 0
		else
			iBeginReal = iBegin
		endif

		if (iEnd == -1) then
			iEndReal = iLength / sr
		elseif (iEnd * sr) > iLength then
			prints "\nWARNING m_loop_play %s: loop end too late, setting to end of sample", Sname
			iEndReal = iLength / sr
		else
			iEndReal = iEnd
		endif

		iBaseFreq = 1/(iEndReal - iBeginReal)

		; Produce the output
		if (iRepeat == 0) || ((iRepeat >0) && (kCounter < iRepeat)) then
			aOut flooper2 1, kPitch, iBeginReal, iEndReal, 0, ifn, \
				iBeginReal, iMode, 0, 0
			aPhasor, aSyncOut syncphasor (kPitch * iBaseFreq), aSyncIn
		else ; no more output
			aOut = 0
		endif
	endif ; Loop validity

	; Output the audio
	xout aOut
endop
